#include "./message_n10n.hpp"

#include <solanaceae/message3/components.hpp>

#include <wintoastlib.h>

#include <locale>
#include <codecvt>

class OurHandler : public WinToastLib::IWinToastHandler {
	public:
		virtual ~OurHandler(void) {
		}

	protected: // wintoast
		void toastActivated(void) const override {
			std::cout << "action called\n";
		}

		void toastActivated(int actionIndex) const override {
			std::cout << "action called\n";
		}

		void toastDismissed(WinToastDismissalReason state) const override {
		}

		void toastFailed(void) const override {
		}
};


MessageN10n::MessageN10n(RegistryMessageModel& rmm) : _rmm(rmm) {
	// Register WinToast App User Model
	WinToastLib::WinToast::instance()->setAppName(L"Tomato");
	const auto aumi = WinToastLib::WinToast::configureAUMI(L"green", L"solanaceae", L"solanaceae_message_n10n", L"20240517");
	WinToastLib::WinToast::instance()->setAppUserModelId(aumi);

	// Initialize WinToast
	if (!WinToastLib::WinToast::instance()->initialize()) {
		std::wcout << L"Error, could not initialize the lib!" << std::endl;
	} else {
		_rmm.subscribe(this, RegistryMessageModel_Event::message_construct);
	}
}

MessageN10n::~MessageN10n(void) {
}

bool MessageN10n::onEvent(const Message::Events::MessageConstruct& e) {
	std::cout << "message constructed called\n";

	if (!e.e.all_of<Message::Components::MessageText>()) {
		return false;
	}

	auto templ = WinToastLib::WinToastTemplate(WinToastLib::WinToastTemplate::Text02);
	templ.setTextField(L"title", WinToastLib::WinToastTemplate::FirstLine);
	templ.setTextField(
		std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>>{}.from_bytes(
			e.e.get<Message::Components::MessageText>().text
		),
		WinToastLib::WinToastTemplate::SecondLine
	);

	WinToastLib::WinToast::WinToastError error;
	const auto toast_id = WinToastLib::WinToast::instance()->showToast(templ, new OurHandler, &error);
	if (toast_id < 0) {
		std::wcout << L"Error: Could not launch your toast notification! " << error << std::endl;
	}

	return false;
}
